#define _ZETA_TS_FIND_DIR_
#include "SkinSelectorPlugin.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <AppKit.h>
#include <Debug.h>
#include <InterfaceKit.h>
#include <Directory.h>
#include <File.h>
#include <Path.h>

#define WHAT_SKIN 'skin'

// CL-Amp vars
extern //const 
struct skin_descriptor {
	uint32 dummy1;
	char name[0x10c-4];
} Skins;
//extern void *_7Bitmaps.mainSkinsDir;

//BMessage(0x000000C8) close Playlist
//BMessage(0x000000C9) open Playlist
//BMessage(0x0000012D) close Eq
//BMessage(0x0000012C) open Eq
//BMessage(0x00000031) WindowShade mode ?
//BMessage(0x00000012) toggle double mode ?
//BMessage(0x00000037) toggle always on top mode ?
//BMessage(19) // open skin list window
//BMessage(100) // change skin

#define CLAMP_MAIN_PREF_FILE "CL-Amp/CL-Amp.preferences"

SkinSelectorPlugin *SkinSelectorPlugin::inst = NULL;

status_t SkinSelectorPlugin::CurrentSkin(BString *to)
{
	BPath thePath(Skins.name);
	status_t err;
	if (!to)
		return EINVAL;
	if (Skins.name[0] == '\0') { // just got loaded, so no skin path is here... get it from the pref file
		PRINT(("SkinSelector: Using pref file\n"));
		char skinpath[B_PATH_NAME_LENGTH+1];
		BPath settingsPath;
		if (find_directory(B_USER_SETTINGS_DIRECTORY, &settingsPath) != B_OK)
			return B_ERROR;
		settingsPath.SetTo(settingsPath.Path(), CLAMP_MAIN_PREF_FILE);
		BFile settingsFile(settingsPath.Path(), B_READ_ONLY);
		if ((err = settingsFile.InitCheck()) != B_OK) {
			PRINT(("SkinSelector: error 0x%08lx in BFile::InitCheck()\n", err));
			return B_ERROR;
		}
		if ((err = settingsFile.ReadAt(8, skinpath, B_PATH_NAME_LENGTH-1)) <= 0) {
			PRINT(("SkinSelector: error 0x%08lx in BFile::ReadAt()\n", err));
			return B_ERROR;
		}
		thePath.SetTo(skinpath);
	}
	*to << thePath.Path();
	return B_OK;
}

status_t SkinSelectorPlugin::HandleSkinMessage(BMessage *message)
{
	BMessage reply;
	BString skinname;
	status_t err;
	if (!message)
		return EINVAL;
	if (message->FindString("name", &skinname) < B_OK) {
		fprintf(stderr, "%lx, %s\n", Skins.dummy1, Skins.name);
		/* asked the current name */
		err = CurrentSkin(&skinname);
		if (err < B_OK)
			reply.AddInt32("result", err);
		else
			reply.AddString("result", skinname);
		message->SendReply(&reply);
		return B_OK;
	}
	if (strlen(skinname.String()) > B_PATH_NAME_LENGTH)
		return B_NAME_TOO_LONG;
	strcpy(Skins.name, skinname.String());
	BMessage m(100);
	BMessenger msgr(GetCLAmpHandler());
	msgr.SendMessage(&m);
	reply.AddInt32("result", B_OK);
	message->SendReply(&reply);
	return B_OK;
}

// The first string is intended to be a short label for the type of audio this plugin handles
// The second string should give a more complete description of the plugin
SkinSelectorPlugin::SkinSelectorPlugin(struct PluginConstructorStruct *ConstrInfo) : GeneralPlugin ("skin-selector", "Skin Selector plugin")
{
	PRINT(("SkinSelectorPlugin::SkinSelectorPlugin()\n"));
	fMFilter=NULL;
	inst = this;
}


filter_result SkinSelectorPlugin::FilterFunc(BMessage *msg, BHandler **target, BMessageFilter *filter)
{
	if (Instance())
		Instance()->HandleSkinMessage(msg);
	return B_DISPATCH_MESSAGE;
}

bool SkinSelectorPlugin::Init()
{
	PRINT(("SkinSelectorPlugin::Init()\n"));
	fMFilter = new BMessageFilter(B_ANY_DELIVERY, B_ANY_SOURCE, WHAT_SKIN, SkinSelectorPlugin::FilterFunc);
	be_app->Lock();
	be_app->AddCommonFilter(fMFilter);
	be_app->Unlock();
	return true;
}

void SkinSelectorPlugin::Cleanup()
{
	status_t ret;
	bool succeeded = true;
	
	PRINT(("SkinSelectorPlugin::Cleanup()\n"));

	/* CL-Amp plays dirty on exit... */
	if (fMFilter && (be_app->LockWithTimeout(10000) == B_OK)) {
		be_app->RemoveCommonFilter(fMFilter);
		be_app->Unlock();
	}
		
	if (fMFilter)
		delete fMFilter;
	fMFilter = NULL;
}

unsigned long SkinSelectorPlugin::GetFlags()
{
	PRINT(("SkinSelectorPlugin::GetFlags()\n"));
	return 0;
}

bool SkinSelectorPlugin::About(bool Question)
{
	if (!Question) {

		BAlert *theAlert = new BAlert("",
				"SkinSelector\nVersion 0.0.1\n\nMade by François Revol\nE-mail: revol@free.fr\n",
				"Oh-Kay !!!", NULL, NULL, B_WIDTH_FROM_LABEL);

		BTextView *theText = theAlert->TextView();
		if(theText) {
			theText->SetStylable(true);
			theText->Select(0,12);	// Adjust this to the length of your header!
									// (The header in this example is "VU-Meter")
			BFont ourFont;

			theText->SetFontAndColor(be_bold_font);
			theText->GetFontAndColor(2, &ourFont, NULL);
			ourFont.SetSize(24);
			theText->SetFontAndColor(&ourFont);
		}
		theAlert->Go();
	}
	return (true);
}

bool SkinSelectorPlugin::Prefs(bool Question)
{
	return false;
}

